package ch.epfl.ppo;

import java.util.ArrayList;
import java.util.List;

public final class LZWConcreteDecoder implements LZWDecoder {
    private final List<String> dict;
    private final int dictCapacity;

    public LZWConcreteDecoder(String alphabet, int dictCapacity) {
        if (! (dictCapacity >= alphabet.length()))
            throw new IllegalArgumentException("dict. capacity smaller than alphabet");

        List<String> dict = new ArrayList<>(dictCapacity);
        for (int i = 0; i < alphabet.length(); ++i)
            dict.add(alphabet.substring(i, i + 1));

        this.dict = dict;
        this.dictCapacity = dictCapacity;
    }

    @Override
    public String decode(List<Integer> l) {
        StringBuilder r = new StringBuilder();
        String p = null;
        for (int i: l) {
            String s = (i < dict.size() ? dict.get(i) : p + p.charAt(0));
            if (dict.size() < dictCapacity && p != null)
                dict.add(p + s.charAt(0));
            r.append(s);
            p = s;
        }
        return r.toString();
    }
}
